Package: mstATA
Title: Automated Test Assembly for Multistage Tests Using Mixed-Integer
        Linear Programming
Version: 0.1.0
Authors@R: person("Hong","Chen", email="hongchen030@gmail.com", role = c("aut", "cre"))
Description: Provides a suite of mixed-integer linear programming (MILP) model builders and solvers—including 'Gurobi', 'HiGHS', 'Symphony', 'GNU Linear Programming Kit (GLPK)', and 'lpSolve'—for automated test assembly (ATA) in multistage testing (MST). Offers filtering of decision variables through item–module eligibility and the application of explicit bounds to simplify the MILP model and accelerate the optimization process. Supports bottom up, top down, and hybrid assembly strategies; enemy-item and enemy-stimulus exclusions; stimulus all in/all out or partial selection; anchor item/stimulus specification; and item exposure control. Accommodates both single-objective and multi-objective optimization ('weighted sum', 'maximin', 'capped maximin', 'minimax', and 'goal programming'). Enables simultaneous assembly of multiple panels with item and stimulus content balancing and exposure control. Provides analytical evaluation of assembled MST performance within seconds. Includes tools for diagnosing infeasible optimization models by systematically identifying sources of infeasibility and reformulating models with slack variables to restore feasibility.Methods implemented in this package build on established work in optimal test assembly (van der Linden, 2005 <doi:10.1007/0-387-29054-0>), item-set constrained test assembly (van der Linden, 2000 <doi:10.1177/01466210022031697>), hybrid assembly (Xiong, 2018 <doi:10.1177/0146621618762739>), recursion-based analytic methods (Lim et al., 2021 <doi:10.1111/jedm.12276>), and classification evaluation (Rudner, 2000 <doi:10.7275/an9m-2035>; Rudner, 2005 <doi:10.7275/56a5-6b14>).
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0)
LazyData: true
Config/testthat/edition: 3
Imports: stats, utils, dplyr, rlang, methods, Matrix, ggplot2
Suggests: testthat (>= 3.0.0),knitr,rmarkdown,
        kableExtra,highs,lpSolveAPI,Rglpk,Rsymphony,irtQ,mstR,numDeriv,ggformula,ggpubr
VignetteBuilder: knitr
URL: https://github.com/Hongchen030/mstATA
BugReports: https://github.com/Hongchen030/mstATA/issues
NeedsCompilation: no
Packaged: 2026-03-18 15:24:07 UTC; hongchen
Author: Hong Chen [aut, cre]
Maintainer: Hong Chen <hongchen030@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-23 10:00:02 UTC
Built: R 4.6.0; ; 2026-04-29 05:22:38 UTC; unix
